@@ -1,8 +1,8 @@
 /*
  *******************************************************************************
  *
- *   Copyright (C) 2016 and later: Unicode, Inc. and others.
- *   License & terms of use: http://www.unicode.org/copyright.html#License
+ *   © 2016 and later: Unicode, Inc. and others.
+ *   License & terms of use: http://www.unicode.org/copyright.html
  *
  *******************************************************************************
  *******************************************************************************
@@ -23,6 +23,6 @@
 
 void GDIGUISupport::postErrorMessage(const char *message, const char *title)
 {
-    MessageBoxA(NULL, message, title, MB_ICONERROR);
+    MessageBoxA(nullptr, message, title, MB_ICONERROR);
 }
 
